/*
iplcd (c) Copyright Neil Cherry 2005
Portions of this code have been taken from x10d.c and are (c) Copyright
Daniel D. Lanciani 1995-2005

All rights reserved.
 
iplcd is licensed free of charge for non-commercial distribution and for
personal and internal business use only.  iplcd may not be distributed
for profit, nor may it be included in products or otherwise distributed by
commercial entities to their clients or customers without the prior written
permission of the authors.
 
TO THE EXTENT ALLOWED BY APPLICABLE LAW, iplcd IS PROVIDED
"AS IS", WITH NO EXPRESS OR IMPLIED WARRANTY, INCLUDING, BUT
NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.
 
IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL THE AUTHOR BE
LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO
USE iplcd EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH DAMAGES.
 
These copyright, license, and disclaimer notices must be included
with all copies of iplcd.  Modified versions of iplcd must be
marked as such.

ddl@danlan.com
*/


/*
** Author:	Neil Cherry
** Date:	11/25/99
** Version:	1.6 (alpha)
**
**      $Id: fini.c,v 1.6 1999/11/25 17:32:13 njc alpha njc $
**
**      $Log: fini.c,v $
**      Revision 1.6  1999/11/25 17:32:13  njc
**      This code handle the shutdown of the daemon. Later it will contain the
**      code to clean up the uucp locking files plus whatever else may need to
**      be tidied up.
**
*/

#include <stdlib.h>
#include <syslog.h>
#include <signal.h>

#ifdef UUCP
#include <unistd.h>

extern char *lock;
#endif

extern int clprintf(const char *format, ...);

void
fini(int sig)
{
  /*
  ** I'll need to add the code to remove the uucp lock file
  ** as well as shutting down the daemon cleanly
  */
  clprintf("A \r\n"); // Tells the user we're aborting

  /*
  ** Done!
  */

  switch(sig) {
    case SIGHUP:
      syslog(LOG_DEBUG, "Fini");
      break;
      
    case SIGSEGV:
      syslog(LOG_DEBUG, "SIGSEGV");
      break;
      
    case SIGFPE:
      syslog(LOG_DEBUG, "SIGFPE - Floating point error");
      break;
      
    case SIGABRT:
      syslog(LOG_DEBUG, "SIGABRT - Core dump");
      break;

    case SIGTERM:
      syslog(LOG_DEBUG, "Terminated");
      break;

    default:
      syslog(LOG_DEBUG, "Unexpected signal (%d)", sig);
      break;
  }

#ifdef UUCP
  unlink(lock);
  free(lock);
#endif
  
  exit(0);
}

