/*
iplcd (c) Copyright Neil Cherry 2005
Portions of this code have been taken from x10d.c and are (c) Copyright
Daniel D. Lanciani 1995-2005

All rights reserved.
 
iplcd is licensed free of charge for non-commercial distribution and for
personal and internal business use only.  iplcd may not be distributed
for profit, nor may it be included in products or otherwise distributed by
commercial entities to their clients or customers without the prior written
permission of the authors.
 
TO THE EXTENT ALLOWED BY APPLICABLE LAW, iplcd IS PROVIDED
"AS IS", WITH NO EXPRESS OR IMPLIED WARRANTY, INCLUDING, BUT
NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.
 
IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL THE AUTHOR BE
LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO
USE iplcd EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH DAMAGES.
 
These copyright, license, and disclaimer notices must be included
with all copies of iplcd.  Modified versions of iplcd must be
marked as such.

ddl@danlan.com
*/


/*
** Author:	Neil Cherry
** Date:	11/25/99
** Version:	1.6 (alpha)
**
**      $Id$
**
**      $Log$
**
*/

#include <errno.h>
#include <unistd.h>
#include <syslog.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <string.h>
#include <stdlib.h>

#include "defs.h"
#include "clients.h"
#include "types.h"

int waitfor_rd(int fd, int nsec, int usec);
int waitfor_wr(int fd, int nsec, int usec);

void clswrstr(const char *wbuf, int count);
int  clwrite(int client, const char *str, int size);


/*
** This will give me the option to snoop
**
** xread will loop until it get read count
** characters. which it simply puts into buf
*/

/*
** Dan does the following:
**
** FD_SET(n, &xprobe);
**
** tv.tv_sec  = 0;
** tv.tv_usec = count*1500;     // We get a character ~once/1500 usec
**
** c = select(n + 1, &xprobe, FD_NULL, FD_NULL, &tv);
**
** if(c < 0) {
**   syslog(LOG_ERR, "select addrreceive: %m");
**   return (-1);
** }
**
** if(c == 0) {
**   syslog(LOG_ERR, "select addrreceive timeout");
**   return (-1);
** }
** c = read(n, buf + cnt, 2 - cnt);
**
** This would be a good way for me to cut back on endless looping or read hangs
** (good for slower machines).
**
** This is a future item for now.
*/


ssize_t
xread(int fd, char *rbuf, size_t count) {
  int i, j, k, x, usec;

  char *ptr;

  i = 0;
  k = 0;

  usec = count * 2000000;
  
  /*
  ** Malloc the memory
  */

  if((ptr = malloc(count+4)) == NULL) {
    fprintf(stderr, "XPRESS malloc error (%d)\n", errno);
    syslog(LOG_CRIT, "XPRESS malloc error: %m");
    return((-1));
  }

  /*
  ** count down
  */

  while(i < count) {

    /*
    ** Here is where the select code belongs. We can figure out the time
    ** it will take to get x bytes at 9600 (assumed because we're dealing
    ** with the serial port as the slowest device).
    **
    ** This should allow us to correctly handle timeouts and other read mishaps
    */

    j = 0;

    /*
    ** retry 3 time, then out
    */

    while(((k = waitfor_rd(fd, 0, usec)) <= 0) && (j < 3)) {
      j++;
    }
    

    if(k < 1) {
      *rbuf = '\0';
      syslog(LOG_ERR, "wait_for() < 1");
      return (-1);
    }
    
    if((j = read(fd, ptr, count)) > 0) {
      for(x = 0; x < j; x++) {
        *rbuf = *(ptr+x);
        rbuf++;
        i++;
      }
    } else {
      if(!(j == 0 || errno == EAGAIN)) {
        syslog(LOG_ERR, "(%d) Read error (%d)", j, errno);
        free(ptr);
        return((-1));
      }
    }
  }
  free(ptr);

  return(i);
}

extern int nclient;
extern Client *clients;

extern char *hexstr;

ssize_t
clswrite(const char *wbuf, size_t count) {
  clswrstr(wbuf, count);
  
  return(count);
}

// Write to every client
void
clswrstr(const char *wbuf, int count) {
  int i;
  
  for(i = 0; i < nclient; i++) {
    if(clients[i].quiet)
      continue;
    
    if(clwrite(clients[i].socket, wbuf, count) != count)
      shutdown(clients[i].socket, 2);
  }
}

/*
** I wrote this because I am getting too lazy to keep typing in the same
** lines. Besides this reads better when viewing the code. Readability is
** important too.
*/

int
clprintf(const char *format, ...) {
  va_list ap;

  char p[MAXLINE+1];
  char *ptr;
  int  j, x;

  ptr = p;

  va_start(ap, format);
  j = vsnprintf(ptr, MAXLINE, format, ap);
  va_end(ap);
  if( j < 0 ) {
    syslog(LOG_ERR, "clprint j is less than 0");
    return(-1);
  }

  if(j) {
    clswrstr(ptr, j);
    x = j;
  } else {
    syslog(LOG_INFO, "fdprintf nothing to write");
    x = 0;
  }
  return(x);
}

int
fdprintf(int fd, const char *format, ...) {
  va_list ap;

  char p[MAXLINE+1];
  char *ptr;
  int  j, x;

  ptr = p;

  va_start(ap, format);
  j = vsnprintf(ptr, MAXLINE, format, ap);
  va_end(ap);
  if( j < 0 ) {
    syslog(LOG_ERR, "fdprint j is less than 0");
    return(-1);
  }

  if(j) {
    x = write(fd, ptr, j);
  } else {
    syslog(LOG_INFO, "fdprintf nothing to write");
    x = 0;
  }
  return(x);
}


/*
** Works like perror, send the error string to the client
** only this sends the error message to each connected client
*/
int
clperror(const char *format, ...) {
  va_list ap;

  char *p;
  char t[MAXLINE+1];

  char *x;

  int  j;

  if ((p = malloc (MAXLINE+1)) == NULL)
    return 0;

  x = strerror_r(errno, t, MAXLINE);

  va_start(ap, format);
  j =  vsnprintf(p, MAXLINE, format, ap);
  va_end(ap);

  if( j < 0 ) {
    syslog(LOG_ERR, "fdprint j is less than 0");
    return(j);
  }

  clprintf("[%s] :(%s)\r\n", p, x);
  //syslog(LOG_ERR, "%s :%s ", p, x);

  free(p);
  return(j);
}

/*
** Works like perror, send the error string to the client
** only this sends the error message to each connected client
**
** Actually doens't work and I don't know why ???
*/
int
fdperror(int fd, const char *format, ...) {
  va_list ap;

  char p[MAXLINE+1];
  char t[MAXLINE+1];

  char *x;

  int  j;

  x = strerror_r(errno, t, MAXLINE);

  va_start(ap, format);
  j =  vsnprintf(p, MAXLINE, format, ap);// Broken, p is null, so is format

  if( j < 0 ) {
    syslog(LOG_ERR, "fdprint j is less than 0");
    va_end(ap);
    return(j);
  }

  syslog(LOG_ERR, "%s :%s ", p, x);

  va_end(ap);
  return(j);
}

int
dwrite(int device, u_char *str, int size) {
  int i = 0;

  u_char cmd[MAXLINE];             /* Later Malloc this! */

  // Strncpy bad, it's not a string!
  //strncpy(cmd, str, size);
  memcpy(cmd, str, size);

  i = write(device, cmd, size);


  return(i);
}


int
clwrite(int client, const char *str, int size) {
  int i = 0;

  //  char cmd[MAXLINE];             /* Later Malloc this! */

  //  strncpy(cmd, str, size);

  i = write(client, str, size);

  return(i);
}

/*
** Just return whatever select returns
*/

int
waitfor_rd(int fd, int nsec, int usec) {
  int c;
  
  struct timeval tv;

  fd_set lprobe;

  FD_SET(fd, &lprobe);
  tv.tv_sec  = nsec;             /* wait for n seconds */
  tv.tv_usec = usec;             /* wait for n useconds */

  c = select(fd + 1, &lprobe, FD_NULL, FD_NULL, &tv);
  if(c < 0) {
    syslog(LOG_ERR, "select read error: %m");
  }

  return(c);
}

int
waitfor_wr(int fd, int nsec, int usec) {
  int c;
  
  struct timeval tv;
  fd_set lprobe;

  FD_SET(fd, &lprobe);
  tv.tv_sec  = nsec;             /* wait for n seconds */
  tv.tv_usec = usec;             /* wait for n useconds */

  c = select(fd + 1, FD_NULL, &lprobe, FD_NULL, &tv);
  if(c < 0) {
    syslog(LOG_ERR, "select write error: %m");
  }

  return(c);
}

/*
** readn - read n bytes
**
** We have read setup in non-blocking mode this means it won't wait for bytes
** that aren't there. So If I do a read of 9 bytes it will simply read what
** it has and come back a few bytes short. What this read is to read 1 byte at
** a time and if it gets 2 timeouts return with whatever it's read.
*/
int
readn(int device, char *buf, size_t wanted) {
  int rtn = 0;
  int cnt = 3;
  int iread = 0;       // really means read but that's already defined

  int nsec = 0;
  int usec = 2080;	  // 4800 = 208 us/bit,multi by 10 bits = 2080

  while( iread < wanted) { // we're not done until what iread == wanted
    // wait for something to read
    waitfor_rd(device, nsec, usec);
    // Read character (Msg type)
    do {
      errno = 0;
      if((( rtn = read(device, buf+iread, 1)) < 1) || errno != 0) {
	if(errno != 11) {
	  // We ignore timeouts, happens if we have no data
	  clprintf("\r\nE *** <ERR> (%d/%d) ", rtn, errno);
	  clperror("\r\nE *** Read error (0x%02x)", *buf);
	}
      } else {
	iread++;
	buf[iread] = 0;		// can we move this to before the return ???
	break;
      }
    } while(--cnt);
  }
  return iread;
}
