/*
iplcd (c) Copyright Neil Cherry 2005
Portions of this code have been taken from x10d.c and are (c) Copyright
Daniel D. Lanciani 1995-2005

All rights reserved.
 
iplcd is licensed free of charge for non-commercial distribution and for
personal and internal business use only.  iplcd may not be distributed
for profit, nor may it be included in products or otherwise distributed by
commercial entities to their clients or customers without the prior written
permission of the authors.
 
TO THE EXTENT ALLOWED BY APPLICABLE LAW, iplcd IS PROVIDED
"AS IS", WITH NO EXPRESS OR IMPLIED WARRANTY, INCLUDING, BUT
NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.
 
IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL THE AUTHOR BE
LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO
USE iplcd EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH DAMAGES.
 
These copyright, license, and disclaimer notices must be included
with all copies of iplcd.  Modified versions of iplcd must be
marked as such.

ddl@danlan.com
*/


/*
**
*/

#ifndef INSTEON_H

#define INSTEON_H

#include "types.h"

#define STX		0x02
#define ETX		0x03
#define ACK		0x06
#define NL		0x0a // ^J I don't need this one but it keeps my head straight
#define CR		0x0d // ^M (we no longer need this one either
#define NAK		0x15

#define INTL_CNT	0x0024
#define RD_H		0x0026
#define RD_L		0x0027

// Command

#define CMD_DOWNLOAD		0x40
#define CMD_TEXT_LEN		0x41
#define CMD_UPLOAD		0x42
#define CMD_TEXT		0x43
#define CMD_GETCHECKSUM		0x44
#define CMD_EVENTREPORT		0x45
#define CMD_MASK		0x46
#define CMD_SIMULATEDEVENT	0x47
#define CMD_GETVERSION		0x48
#define CMD_DEBUGREPORT		0x49
#define CMD_X10			0x4A
//#define CMD_			0x4B
//#define CMD_			0x4C
//#define CMD_			0x4D
//#define CMD_			0x4E
#define CMD_INSTEON		0x4F

// Event Handling - Event table

#define EVNT_INIT	0x00	// Executes initialization code (automatic at power-up)
#define EVNT_IRX_MYMSG	0x01	// First (unique) message received for this unit
#define EVNT_IRX_MSG	0x02	// First (unique) message received in a Hop Sequence
#define EVNT_IRX_PKT	0x03	// Message received
#define EVNT_ITX_ACK	0x04	// Insteon Acknowledge received
#define EVNT_ITX_NACK	0x05	// Insteon Acknowledge not received (timeout)
#define EVNT_IRX_BADID	0x06	// Insteon Unenrolled ID
#define EVNT_IRX_ENROLL	0x07	// Insteon Enrollment triggered by enrollment comands
				// (i.e. "Join Group" or "Button Pushed"). Only occurs
				// after the ENROLL instruction has been successfully
				// executed.
#define EVNT_XRX_MSG	0x08	// X10 Message Received
#define EVNT_XRX_XMSG	0x09	// X10 Executed Message Received
#define EVNT_BTN_TAP	0x0A	// Push Button tap (where Button Tap Time = 0, Button
				// Tap Count = # of taps, see Flat Memory Model
#define EVNT_BTN_HOLD	0x0B	// Push Button Hold
#define EVNT_BTN_REL	0x0C	// Push Button Release (from Hold)
#define EVNT_TICK	0x0D	// Tick counter has expired (NTL_TICK)
#define EVNT_ALARM	0x0E	// Hours and minutes equals current time
#define EVNT_MIDNIGHT	0x0F	// Event occurs every midnight
#define EVNT_2AM	0x10	// Event occurs every 2:00 AM
#define EVNT_RX		0x11	// Received a serial byte for SALad processing
#define EVNT_SRX_COM	0x12	// Serial received a command
#define EVNT_DAUGHTER	0x13	// Event Daughter card interrupt

// I2C Addr register

// Status registers

// Flat memory model

// UAM - Universal address module

// X10 (Insteon related)

// X10_Flags 
#define _X10_RTS	0x80 // 1000 0000
#define _X10_TXEX	0x40 // 0100 0000
#define _X10_EXTENDED	0x20 // 0010 0000
#define _X10_COMBUF	0x10 // 0001 0000
#define _X10_TXCOMMAND	0x08 // 0000 1000
#define _X10_RXCOMMAND	0x04 // 0000 0100
#define _X10_VALID	0x02 // 0000 0010
#define _X10_ENABLED	0x01 // 0000 0001

// X10 command bit

// Set the bit on ex: 0x02 addr
#define SetFlag(value, flag)	(value || flag)
#define ClrFlag(value, flag)	(value && ~flag)
#define ClearFlag(value, flag)	(value && ~flag)


// Returns true if flag bit is set false of not
#define TestFlag(value, flag)	(value && flag)

// X10 commands and addresses are 8 bits wide. By setting the 16th bit
// I can safely use this on shorts (16 bits) and longs (32 bits)
#define X10CmdFlag 0x00008000

#define isX10Cmd(value)		(value && X10CmdFlag)
// Mask off all but the lower 8 bits
#define X10(value)		(value && 0x000000ff)

// These are not ready for prime time yet

struct StandardMessage {
  byte from[3];
  byte to[3];

  byte flags; // bit field
  byte cmds[2];
};

struct ExtendedMessage {
  byte from[3];
  byte to[3];

  //byte flags; // bit field
  struct flags_struct {
    byte mtype1:3; // Message type
    byte xflag:1;  // Extended flag
    byte HopsC:2;  // Hops Remaining (decremented during each hop)
    byte HopsT:2;  // Hops Total
  } flags;
  byte cmds[2];
  byte data[14];
};


#endif // INSTEON_H
