/*
iplcd (c) Copyright Neil Cherry 2005
Portions of this code have been taken from x10d.c and are (c) Copyright
Daniel D. Lanciani 1995-2005

All rights reserved.
 
iplcd is licensed free of charge for non-commercial distribution and for
personal and internal business use only.  iplcd may not be distributed
for profit, nor may it be included in products or otherwise distributed by
commercial entities to their clients or customers without the prior written
permission of the authors.
 
TO THE EXTENT ALLOWED BY APPLICABLE LAW, iplcd IS PROVIDED
"AS IS", WITH NO EXPRESS OR IMPLIED WARRANTY, INCLUDING, BUT
NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.
 
IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL THE AUTHOR BE
LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO
USE iplcd EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH DAMAGES.
 
These copyright, license, and disclaimer notices must be included
with all copies of iplcd.  Modified versions of iplcd must be
marked as such.

ddl@danlan.com
*/


/*
** iplcd.c - A daemon that uses a TCP socket to allow bidirectional
**           communication to the Insteon/Smarthome Powerlinc V2 plc
**
** A person (or program) could telnet (use TCP) to connect to the monitored port
** and issue commands to be sent via the power line or RF. Multiple users are
** permitted to connect simultaneously.
**
** Notes:
**
** Author:	Neil Cherry
** Date:	2005/08/28
** Version:	1.0 (sub alpha)
**
**      $Id$
**
**	$Log$
**
**
*/

char vcid[] = "$Id$"; /* Just the ident string   */
char myid[] = "$Revision"; /* Just the version string */

#include <ctype.h>              /* needed for toupper */
#include <fcntl.h>              /* used by open       */
#include <sys/stat.h>           /* used by open       */
#include <sys/ioctl.h>
#include <sys/socket.h>	// socket()
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <netdb.h>	// getservbyname()
#include <stdio.h>
#include <stdlib.h>	// malloc() atoi()
#include <stdarg.h>
#include <signal.h>
#include <syslog.h>	// Needed by openlog()
#include <string.h>

#if 0
/* According to POSIX 1003.1-2001, select()*/
#include <sys/select.h>
#else
/* According to earlier standards, select() */
#include <sys/time.h>
#include <sys/types.h>	// also socket()
#include <unistd.h>
#endif

#include "defs.h"
#include "clients.h"
#include "iplcd.h"

extern int  setup(char *line);
extern void setclock(int device);
extern void parse_device(int device);
extern void new_client(fd_set *rset, int s);
extern void existing_client(fd_set *rset, fd_set *probe, Client *clients, int device);

/*[ Code starts here ]*******************************************************/

struct sockaddr_in s_in;

int
main(int argc, char **argv) {
  int device, i;
  int nport, tport;

  struct servent *sp;
  fd_set rset, probe;

  // Process command line options
  options(argc, argv);

  openlog(X10D, LOG_PID, LOG_LOCAL0);

  syslog(LOG_INFO, "%s start", port);

  /*
  ** Here is where we handle all the setup for the hcsd
  ** We also handle the opening up of the port
  */

  device = setup(line);

  /*
  */

  // This is Dan's code.
  if(!(clients = (Client *) malloc(maxclient * sizeof(struct client)))) {
    syslog(LOG_CRIT, "no memory for clients");
    exit(errno);
  }

  for(i = 0; i < maxclient; i++) {
    clients[i].socket = (int) -1;
    clients[i].cnt    = 0;
    clients[i].perm   = 0;
    clients[i].buf[0] = 0;
    clients[i].id[0]  = 0;
    clients[i].uid    = -1;
    clients[i].debug  = 0;
    clients[i].quiet  = 0;
  }

  // This is Dan's code (2 lines)
  FD_ZERO(&rset);
  FD_ZERO(&xprobe);

  // the port can be a name or a number
  if(isdigit(*port)) {
    // If it's a number we need to convert it from a string to a
    // number (in the correct network format!).
    tport = atoi(port);
    nport = htons(tport);

    //convert port string to port number
    if(!(sp = getservbyport(nport, "tcp"))) {
      // Need to fill in the blanks and make our own sp
      if(tport < 65536 && tport > 0) {
	s_in.sin_port   = nport; //	sp->s_port = nport;
      } else {
	syslog(LOG_ERR, "unknown service number: %d", tport);
	exit(errno);
      }
    }else {
	s_in.sin_port   = sp->s_port;
    }

  } else {
    // Otherwise it's a string
    if(!(sp = getservbyname(port, "tcp"))) {
      syslog(LOG_ERR, "%s: unknown service port", port);
      exit(errno);
    }
    s_in.sin_port   = sp->s_port;
  }

  /*
  ** this section (socket/bind/listen) was stolen from Dan Lanciani's
  ** X10D. I had a hard time modifying it because I couldn't grok
  ** it. Now I have a few pointers so I can rewrite it. I'll hope to
  ** be doing that in the near future!
  */

  s_in.sin_family = PF_INET;
  // This line is Dan's code
  //s_in.sin_port   = sp->s_port; // Moved this to the above

  memset(&(s_in.sin_zero), 0, 8);        /* zero the rest of the struct */

  /*
  ** To accept connections, a socket is first created with socket(2),
  ** a willingness to accept incoming connections and a queue limit
  ** for incoming connections are specified with listen, and then the
  ** connections are accepted with accept(2).  The listen call applies
  ** only to sockets of type SOCK_STREAM or SOCK_SEQPACKET.
  */

  // Dan's code
  if((lsocket = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
    syslog(LOG_ERR, "socket: %m");
    exit(errno);
  }

  // sin should be struct socket_addr * but I have it as struct
  // sockaddr_in * Weird thing is that both structure are byte-wise
  // the same just built differently

  // Dan's code
  if(bind(lsocket, (struct sockaddr *)&s_in, sizeof(address))) { // &sin gives incompatible pointer type, ignore it
    syslog(LOG_ERR, "bind: %m");
    exit(errno);
  }

  // Dan's code
  listen(lsocket, 5);
  /*
  ** FIONBIO ioctl function controls nonblocking I/O on a socket.  The
  ** Enable argument is a pointer to an integer that specifies whether
  ** nonblocking I/O should be enabled or disabled.  A value of 1
  ** enables nonblocking I/O, and a value of 0 disables nonblocking
  ** I/O.  By default, nonblocking I/O is disabled when a socket is
  ** created.
  */
  fcntl(lsocket, F_SETFL, NON_BLOCKING);

  // Dan's code (2 lines)
  // ioctl(lsocket, FIONBIO, NON_BLOCKING);
  FD_SET(lsocket, &rset);

  FD_SET(device, &rset);

  /*
  ** select does the job of 'forking' the process for us. (it's not really
  ** forking) everytime a new client attaches they run a copy of this
  ** process. (need to double check that this is true).
  */
  
  // While this is not Dan's code the subroutines contain Dan's code
  while(TRUE) {
    probe = rset;

    if(select(FD_SETSIZE, &probe, FD_NULL, FD_NULL, TIMEVAL_NULL) <= 0) {
      syslog(LOG_ERR, "select: %m");
      //sleep(2);
      continue;
    }

    /************************************************************************/

    // =[ Message from device ]===============================================
    
    if(FD_ISSET(device, &probe)) {
      parse_device(device);
    }

    // =[ New client starting up ]============================================
    
    if(FD_ISSET(lsocket, &probe)) {
      syslog(LOG_INFO, "New connection");

      new_client(&rset, lsocket);
    } /* End of listening code */
    
    // =[ Message from existing client ]======================================
    
    existing_client(&rset, &probe, clients, device);
    
    /************************************************************************/
  }
}
