/*
iplcd (c) Copyright Neil Cherry 2005
Portions of this code have been taken from x10d.c and are (c) Copyright
Daniel D. Lanciani 1995-2005

All rights reserved.
 
iplcd is licensed free of charge for non-commercial distribution and for
personal and internal business use only.  iplcd may not be distributed
for profit, nor may it be included in products or otherwise distributed by
commercial entities to their clients or customers without the prior written
permission of the authors.
 
TO THE EXTENT ALLOWED BY APPLICABLE LAW, iplcd IS PROVIDED
"AS IS", WITH NO EXPRESS OR IMPLIED WARRANTY, INCLUDING, BUT
NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.
 
IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL THE AUTHOR BE
LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO
USE iplcd EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH DAMAGES.
 
These copyright, license, and disclaimer notices must be included
with all copies of iplcd.  Modified versions of iplcd must be
marked as such.

ddl@danlan.com
*/


/*
**
** Author:	Neil Cherry
** Date:	10/16/99
** Version:	1.1 (sub alpha)
**
** $Id$
**
** $Log$
**
*/

#include <getopt.h>             /* as referenced in the getop man page */
#include <stdlib.h>             /* used by atoi */
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "defs.h"
#include "options.h"

extern char xdate[];

void
clihelp() {
  // OK so I'm fudging it!

  fprintf(stderr,"Help!\n");
  fprintf(stderr,"\n  ** --device -d - device name to connect to\n  **               defaults to %s\n  ** --port -p   - port to listen on\n  **               defaults to %s\n  **               (can use a name or number)\n  ** --ip -i     - only listen on this IP address\n  ** --usb -u    - don't perform the serial port setup.\n  **               for use with the USB PLC\n  ** --nofork -n - don't fork\n  **\n  ** --help -h -?  This help message\n", DEVICE, SERVICE);
}

struct option long_options[] = {
    {"device",	1, 0, HCSDEVICE},
    {"file",	1, 0, EVENTSFILE},
    {"port",	1, 0, PORT},
    {"lock",    1, 0, LOCK},
    {"debug",	1, 0, DEBUG},
    {"verbose",	1, 0, DEBUG},
    {"version",	0, 0, VERSION},
    {"nofork",  0, 0, NOFORK},
    {"usb",     0, 0, myUSB},
    {"ip",	1, 0, IP},
    {"help",	0, 0, HELP},
    {0,		0, 0, 0}
};

void
options(int argc, char **argv) {
  int c, option_index = 0;

  strcpy(line, DEVICE);
  strcpy(port, SERVICE);
  strcpy(hcsfile, HCSFILE);
  
  /*
  ** handle options here before we fork
  **
  ** The following options should be available:
  ** --device -d - device name to connect to
  **               defaults to /dev/hcsd
  ** --port -p   - port to listen on
  **               defaults to hcsd
  ** 
  ** --help -h -?  Issues help
  */

  while(1) {                    /* Forever loop */
    c = getopt_long (argc, argv, "d:s:p:l:x:i:vunh?", long_options, &option_index);

    // getopt_long can return the following:
    //  returns the option character if the option was found successfully
    //  ':'  if  there  was  a  missing parameter for one of the options
    //  '?'  for an unknown option character
    //   -1  for the end of the option list

    switch(c) {
      case EndOfParameters:
	return ;
	break;

      case PORT:                /* The TCP/IP port name or number */
        if(strlen(optarg) < MAXLINE)
          strcpy(port, optarg);
        break;
        
      case HCSDEVICE:           /* the /dev/HCSDEVICE name */
        if(strlen(optarg) < MAXLINE)
          strcpy(line, optarg);
        break;

      case LOCK:
        if(strlen(optarg) < MAXLINE)
          strcpy(lock, optarg);
        break;
        
      case SPEED:               /* The speed at which to communicate */
        speed = atoi(optarg);
        break;

      case DEBUG:
        debug = atoi(optarg);
        if(!debug)
          debug++;
        break;

      case VERSION:
        printf("Version: %s %s\n", myid, xdate);
        exit(EPERM);
        break;

      case NOFORK:
	mydaemon = 0;
	break;

      case IP:
	// Change the IP you listen on
	// 0.0.0.0 will listen on all the ether's of the box
	// some like 192.168.24.250 will listen only on that ethernet interface
	// Not on 127.0.0.1 or any other IP address.
	break;
      case myUSB:
	myusb = 1;
	break;

      default:                  /* Bad option */
        fprintf(stderr,"Bad option '%c'\n",**argv);
      case 'h':                 /* Help screen */
      case HELP:
	clihelp();
        exit(EPERM);
        break;
    }
  }
}
