/*
iplcd (c) Copyright Neil Cherry 2005
Portions of this code have been taken from x10d.c and are (c) Copyright
Daniel D. Lanciani 1995-2005

All rights reserved.
 
iplcd is licensed free of charge for non-commercial distribution and for
personal and internal business use only.  iplcd may not be distributed
for profit, nor may it be included in products or otherwise distributed by
commercial entities to their clients or customers without the prior written
permission of the authors.
 
TO THE EXTENT ALLOWED BY APPLICABLE LAW, iplcd IS PROVIDED
"AS IS", WITH NO EXPRESS OR IMPLIED WARRANTY, INCLUDING, BUT
NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.
 
IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL THE AUTHOR BE
LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO
USE iplcd EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH DAMAGES.
 
These copyright, license, and disclaimer notices must be included
with all copies of iplcd.  Modified versions of iplcd must be
marked as such.

ddl@danlan.com
*/



#include <syslog.h>
#include <sys/types.h>
#include <unistd.h>

#include <errno.h>

#include "defs.h"
#include "x10.h"
#include "iplc.h"

extern void dumphex(char *rbuf1, char *rbuf2, int size2);
extern void clperror(const char *format, ...);
extern int  fdprintf(int fd, const char *format, ...);
extern int  clprintf(const char *format, ...);
extern void setclock(int device);
extern int  waitfor_rd(int fd, int nsec, int usec);
extern int  clprintf(const char *format, ...);
extern void x10read(int device);
extern void download(int device);
extern void EventReport(int device);
extern void text_len(int device);
extern void text(int device);
extern void insteon(int device);
extern void version(int device);

struct client {
  int  socket,                  // clients fd
       cnt,                     // n bytes waiting in queue
       perm;                    // Permissions either 0=R or 1=RW
  char ipaddr[257];		// IP addr of the client
  char buf[MAXLINE];
  char id[MAXLINE];
  int  uid;

  // this is the purpose of the flags about so I might as well use it properly :-)
  int debug_flag:	1;
  int quiet_flag:	1;
};

extern struct client clients[];

extern int errno;

#define _10BITS	2080	 // 4800 = 208 us/bit,multi by 10 bits = 2080

// I think I can safely say that there is none of Dan's code in here

void
parse_device(int device) {
  int rtn;
  int cnt = 3;

  int c;

  /*
  ** OK, this is were things get interesting
  */
  if((c = read(device, rbuf, 1))) {
    /*
    ** This is probably a very good candidate for a rewrite. The reason is
    ** we're not getting the full reponse to commands issued. Only the
    ** $<CMD> (default: case).
    */
        
    switch(MASK(rbuf[0]))  {
    case STX:
      // sleep(1);
      waitfor_rd(device, 0, _10BITS); // 4800 = 208 us/bit,multi by 10 bits = 2080
      do {
	errno = 0;
	if((( rtn = read(device, rbuf+1, 1)) < 1) || errno != 0) {
	  if(errno != 11) {
	    // We ignore timeouts, happens if we have no data
	    clprintf("\r\nE *** (%d/%d) \r\n", rtn, errno);
	    clperror("E *** Read error (0x%02x)", *(rbuf+1));
	  }
	} else {
	  errno = 0;		// Do I really need to do this ???
	  break;
	}
      } while(--cnt);

      if(errno != 0 && errno != 11) {
	clprintf("\r\n");
	clperror("E *** Read error (%d)", errno);
      }
	

      switch(MASK(*(rbuf+1))) {
      case CMD_DOWNLOAD:
	download(device);
	break;

      case CMD_TEXT_LEN:
	clprintf("\r\n<STX> Text Len(0x%02x) ", (u_char) rbuf[1]);
	text_len(device);
	break;

      case CMD_UPLOAD:
	clprintf("\r\n<STX> Upload(0x%02x) ", (u_char) rbuf[1]);
	break;

      case CMD_TEXT:
	clprintf("\r\n<STX> Text(0x%02x) ", (u_char) rbuf[1]);
	text(device);
	break;

      case CMD_GETCHECKSUM:
	clprintf("\r\n<STX> GetChecksum(0x%02x) ", (u_char) rbuf[1]);
	break;

      case CMD_EVENTREPORT:
	// clprintf("\r\n<STX> Event(0x%02x) ", (u_char) rbuf[1]);
	EventReport(device);
	break;

      case CMD_MASK:
	clprintf("\r\n<STX> MASK(0x%02x) ", (u_char) rbuf[1]);
	break;

      case CMD_SIMULATEDEVENT:
	clprintf("\r\n<STX> Simulated(0x%02x) ", (u_char) rbuf[1]);
	break;

      case CMD_GETVERSION:
	version(device);
	break;

      case CMD_DEBUGREPORT:
	clprintf("\r\n<STX> Debug(0x%02x) ", (u_char) rbuf[1]);
	break;

      case CMD_X10:
	//clprintf("\r\n<STX> X10(0x%02x) ", (u_char) rbuf[1]);
	x10read(device);
	break;

      case CMD_INSTEON:
	//clprintf("\r\n<STX> Insteon(0x%02x) ", (u_char) rbuf[1]);
	insteon(device);
	break;

      default:
	clprintf("0x02 0x%02x ", (u_char) rbuf[1]);
	break;	
      }
      break;
      // This fixes a few problems with not actually interpreting the bytes
      // correctly. Not every 0x06 is an ACK or 0x15 a NAK
      //case ACK:
      //clprintf("<0x06> ");
      //break;
      //case NAK:
      //clprintf("<0x15> ");
      //break;
    default:
      clprintf("0x%02x ", (u_char) rbuf[0]);
      break;
    } /* End of switch */
  } else {
    // Bad read rbuf[0]
    if(c < 0) {
      if(errno != EWOULDBLOCK) {
        syslog(LOG_ERR, "read device: %m");
      }
      syslog(LOG_ERR, "C < 0");
    } else {
      syslog(LOG_ERR, "read device: EOF");
    }
  } /* if((c = read(device, rbuf, 1))) */
}
