/*
iplcd (c) Copyright Neil Cherry 2005
Portions of this code have been taken from x10d.c and are (c) Copyright
Daniel D. Lanciani 1995-2005

All rights reserved.
 
iplcd is licensed free of charge for non-commercial distribution and for
personal and internal business use only.  iplcd may not be distributed
for profit, nor may it be included in products or otherwise distributed by
commercial entities to their clients or customers without the prior written
permission of the authors.
 
TO THE EXTENT ALLOWED BY APPLICABLE LAW, iplcd IS PROVIDED
"AS IS", WITH NO EXPRESS OR IMPLIED WARRANTY, INCLUDING, BUT
NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.
 
IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL THE AUTHOR BE
LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO
USE iplcd EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH DAMAGES.
 
These copyright, license, and disclaimer notices must be included
with all copies of iplcd.  Modified versions of iplcd must be
marked as such.

ddl@danlan.com
*/


#include <stdio.h>
#include <string.h>

#include <sys/time.h>
#include <sys/types.h>	// also socket()
#include <unistd.h>
#include <ctype.h>
#include <time.h>

#include "defs.h"
#include "clients.h"
#include "x10.h"
#include "iplc.h"

extern int  fdprintf(int fd, const char *format, ...);
extern int  clprintf(const char *format, ...);
extern int  clwrite(int client, char *str, int size);
extern int  dwrite(int device, u_char *str, int size);
extern void help(int fd);

extern void setclock(int device);
extern void badclient(int fd, fd_set *rset);
extern void dumphex(char *str, u_char *buf, int size);
extern void usDelay(int delay);
extern void x10write(int device, u_char *cmd);

extern void SendInsteon(int device, char *str);
extern void SendX10(int device, char *str);
extern void GetVersion(int device);
extern void ping(int device, char *str);

extern void fini(int i);

extern void intelhex(int client, char *str, int device);

extern Client *clients;

extern char myid[];
extern char *lock;

extern char xdate[];

extern int nclient;

char dbgstr[MAXLINE];

/*
** GPL stuff
*/

void
license(int client) {
  int i = 0;

  char *text[] = {
    "iplcd (c) Copyright Neil Cherry 2005 & Daniel D. Lanciani 2005",
    "Portions of this code have been taken from x10d.c and are (c) Copyright",
    "Daniel D. Lanciani 1995-2005",
    " ",
    "All rights reserved.",
    " ",
    "iplcd is licensed free of charge for non-commercial distribution and for",
    "personal and internal business use only.  iplcd may not be distributed",
    "for profit, nor may it be included in products or otherwise distributed by",
    "commercial entities to their clients or customers without the prior written",
    "permission of the authors.",
    " ",
    "TO THE EXTENT ALLOWED BY APPLICABLE LAW, iplcd IS PROVIDED",
    "\"AS IS\", WITH NO EXPRESS OR IMPLIED WARRANTY, INCLUDING, BUT",
    "NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND",
    "FITNESS FOR A PARTICULAR PURPOSE.",
    " ",
    "IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL THE AUTHOR BE",
    "LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL",
    "OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO",
    "USE iplcd EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY",
    "OF SUCH DAMAGES.",
    " ",
    "These copyright, license, and disclaimer notices must be included",
    "with all copies of iplcd.  Modified versions of iplcd must be",
    "marked as such.",
    " ",
    "ddl@danlan.com",
    " ",
    "",
  };

  while(*text[i]) {
    fdprintf(client, "\r\n%s", text[i]);
    ++i;
  }
}

// =[ Warning!!!!!!!!!! ]=====================================================
// OK, this baby is dangerous! I'm allocation memory local to the functions
// this is OK on a single user system but in a multiuser system it means that
// user a uses the memory and if it's not copied right away to new storage
// user b can come along and stomp on it. Since this function is for debugging
// and I won't have more than 1 user using it that's ok.
// =[ Warning!!!!!!!!!! ]=====================================================

char *
hexstr(char *cmdptr, int len){
  char str[MAXLINE];
  char *p, *s;
  int i = 0;

  p = str;
  s = str;

  for(i = 0; i < len; i++) {
    sprintf(p, "%02x", MASK(*cmdptr++));
    p++; p++;
  }

  return s; // s shuts up the compiler warning about returning local storage
}
// =[ Warning!!!!!!!!!! ]=====================================================

int
stoi(char c) {
  int a;
   
  a = toupper(c);
  if (a < 'G' && a > '@') {
    a = a - 'A' + 10;
  } else if ( a < ':' && a > '/') {
    a = a - '0';
  } else {
    a = 0;
  }
   
  return(a);
}

// !c 07 XX XX XX XX to 0x07 0xXX 0xXX 0xXX 0xXX
// Convert a string (hex like) into a command sent via the
// USB write.

int
hexcmd(int device, char *str) {
  u_char cmd[256];

  int a, i = 0, x;

  if(!(*str)) { // if there's nothing there we're done.
    return  (-1);
  }
  while(*str) {
    if(*str != ' ') {
      a = stoi(*str++) * 16;
      a = stoi(*str++) + a;
      cmd[i] = MASK(a);
      i++;
    } else {
      str++;
    }
  }

  dumphex("***", cmd, i);	// I'll get rid of this later
  x = dwrite(device, cmd, i);
  return x;
}

/*
** User command:
** !w aaaa dd dd dd dd ...
**
** what gets sent:
** 02 40 aaaa llll cccc dd dd dd dd ...
**
** ok what should we do here? I would prefer the we b allowed to read a line at a time
**
*/
void
iwrite(int device, char *str) {
  // 4 byte addr
  // then data
  clprintf("\r\n# *** IWrite: %s", str);
}

/*
** User command:
** !r aaaa llll
**
** What gets sent:
** 02 42 aaaa llll
*/
int
iread(int device, char *str) {
  u_char cmd[256] = { STX, CMD_UPLOAD, 0 };

  int a, i = 2, x;

  if(!(*str)) { // if there's nothing there we're done.
    return  (-1);
  }

  while(*str && (i < 256)) {
    if(isxdigit(*str)) {
      a = stoi(*str++) * 16;
      a = stoi(*str++) + a;
      cmd[i] = MASK(a);
      i++;
    } else {
      if(isspace(*str) != 0) {
	str++; 		// then ignore it.
      } else {
	return(-1);	// Not a hex & not a space then bad!
      }
    }
  }

  dumphex("***", cmd, i);
  x = dwrite(device, cmd, i);
  return x;
}


// Some code in here similar to Dan's (I'm using the same structure names
void
parse_user(char *str, int c, int device, fd_set *rset) {
  char tmp;

  int x; //, len = 0;

  //  char wbuf[10] = { 0,0,0,0,0,0,0 };
  //  char *wptr;
  char *ptr;

  time_t timep;
  time_t *t;

  t = NULL;

  ptr = str;

  if(!str) {
    return;
  }

  /*
  ** This should never happen but for now I'll leave it.
  */
  if(strlen(str) >= MAXLINE) {
    fdprintf(clients[c].socket, "\r\nE *** Command line too long");
    while(*str) str++;          /* eat the rest of the string */
    return ;
  }
  
  while(*str) {
    switch( tmp = toupper(*str)) {
    case xHELP:
      help(clients[c].socket);
      while(*str) str++;
      break;

    case ':':   // Intel hex dump gives us the address, data and checksum (nice)
      intelhex(clients[c].socket, str, device);
      break;

    case '!':	// It's a builtin command
      str++;	// get past the '!'
      switch(toupper(*str)) {

      case xHELP:
	help(clients[c].socket);
	while(*str) str++;
	break;

      case xLICENSE:
	license(clients[c].socket);
	while(*str) str++;
	break;

      case xVERSION:
	GetVersion(device);
	while(*str) str++;
	break;

      case xQUIET:
	// Dan's structure names
	clients[c].quiet = ~clients[c].quiet; /* Client requests no further transmissions */
	while(*str) str++;  /* eat the rest of the string */
	break;
        
      case xEXIT:
	while(*str) str++;  /* eat the rest of the string */
	clwrite(clients[c].socket, "Quit\r\n", 6);
	badclient(c, rset);
	c--;
	return ;
	break;

      case xSTATUS:
	/*
	** Really need to spend a little more time on this code
	*/
	timep =  time(t);
	fdprintf(clients[c].socket, "Version:     %s (%s)\r\n", myid?myid:"?", xdate);
	fdprintf(clients[c].socket, "Local time:  %s\r\n", ctime(&timep) );
	fdprintf(clients[c].socket, "Device file: %s\r\n", line);
	fdprintf(clients[c].socket, "Lock file:   %s\r\n", lock);
        
	fdprintf(clients[c].socket, "%d Clients connected\r\n", nclient);
	while(*str) str++;      /* eat the rest of the string */
	break;

      // =====================================================================
      case xCMD:		/* !c dddddddd.. */
	// !c xx xx xx xx ... (Hex command to send)
	str++;
	if(*str) {
	  x = hexcmd(device, str);
	  fdprintf(clients[c].socket, "Wrote %d bytes\r\n", x);
	  while(*str) str++;      /* eat the rest of the string */
	} else {
	  fdprintf(clients[c].socket, "E *** command lacks paramters\r\n");
	}
	break;

      case xREAD:		/* !r aaaa nnnn */
	str++;
	if(*str) {
	  iread(device, str);
	  while(*str) str++;      /* eat the rest of the string */
	} else {
	  fdprintf(clients[c].socket, "E *** command lacks paramters\r\n");
	}
	break;

      case xWRITE:		/* !w aaaa dddddd... */
	str++;
	if(*str) {
	  iwrite(device, str);
	  while(*str) str++;      /* eat the rest of the string */
	} else {
	  fdprintf(clients[c].socket, "E *** command lacks paramters\r\n");
	}
	break;

      case xIWRITE:		/* !I aaaaaa dddddd... */
	str++;
	if(*str) {
	SendInsteon(device, str);
	  while(*str) str++;      /* eat the rest of the string */
	} else {
	  fdprintf(clients[c].socket, "E *** command lacks paramters\r\n");
	}
	  break;

#if 0
	// X10 ??? we can do a1aon
      case xXWRITE:		/* !# HU dddddd... */
	str++;
	//SendX10(device, str);
	while(*str) str++;      /* eat the rest of the string */
	break;
#endif
      case xPING:
	str++;
	if(*str) {
	  ping(device, str);
	  while(*str) str++;      /* eat the rest of the string */
	// Do I really need to eat the rest of the string? Yes!
	} else {
	  fdprintf(clients[c].socket, "E *** command lacks paramters\r\n");
	}
	break;

      case xDEBUG:
	clients[c].debug = ~clients[c].debug;
	while(*str) str++;      /* eat the rest of the string */
	break;

      case xSHUTDOWN:
	fini(-1);
	break;

      case xUSERS:
	break;

      case xTABLE:
	break;

      case xADDDEVICE:
	break;

      default:
	//syslog(LOG_INFO, "Got default");
	fdprintf(clients[c].socket, "E *** Unknown command %s\r\n", ptr);
	while(*str) str++;      /* eat the rest of the string */
	break;
      }
      break;
    // =[ end of '!' commands ]===============================================

    default:
      // Normal x10d command
      if(!(tmp < 'A' || tmp > 'P')) {
	SendX10(device, str);
	//x10write(device, str);
      } else {
	//syslog(LOG_INFO, "Got default");
	fdprintf(clients[c].socket, "E *** Unknown command %02x (%s)\r\n", *str, ptr);
      }
      while(*str) str++;  /* eat the rest of the string */
      break;
    // =======================================================================
    } 
  }
  fdprintf(clients[c].socket, "\r\nA\r\n");

}
