/*
iplcd (c) Copyright Neil Cherry 2005
Portions of this code have been taken from x10d.c and are (c) Copyright
Daniel D. Lanciani 1995-2005

All rights reserved.
 
iplcd is licensed free of charge for non-commercial distribution and for
personal and internal business use only.  iplcd may not be distributed
for profit, nor may it be included in products or otherwise distributed by
commercial entities to their clients or customers without the prior written
permission of the authors.
 
TO THE EXTENT ALLOWED BY APPLICABLE LAW, iplcd IS PROVIDED
"AS IS", WITH NO EXPRESS OR IMPLIED WARRANTY, INCLUDING, BUT
NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.
 
IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL THE AUTHOR BE
LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO
USE iplcd EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH DAMAGES.
 
These copyright, license, and disclaimer notices must be included
with all copies of iplcd.  Modified versions of iplcd must be
marked as such.

ddl@danlan.com
*/


#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <time.h>

#include <sys/types.h>

#include "types.h"

extern int  clprintf(const char *format, ...);
extern int  msDelay(int t);

extern u_char idcode[];
extern u_char housecode[];

time_t last = 0;

#define ICf(p)  idcode[p]

// OK what is the easiest way to remove this and move it into the
// x10_structures file? For now leave it.

// Basically Dan's code

struct aliases {
  char *name;
  int  len;
  int  code;
} x10aliases[] = {
  { "all units off",    13, 0x00 }, // 
  { "all lights on",    13, 0x01 }, //
  { "on",	         2, 0x02 }, //
  { "off",               3, 0x03 }, //
  { "dim",               3, 0x04 }, //
  { "bright",            5, 0x05 }, //
  { "all lights off",   14, 0x06 }, //
  { "extended code",    13, 0x07 }, //
  { "extended code 1",  15, 0x07 }, // For data/control
  { "hail request",     12, 0x08 }, //
  { "hail acknowledge", 16, 0x09 }, //
  // Preset Dim comments
  // In a Pre-Set Dim instruction, the D8 bit represents the MSB of the
  // level and H1, H2, H8, H8 represent the LSBs.
  { "preset dim 0",     12, 0x0a }, //
  { "preset dim 1",     12, 0x0b }, //
  { "extended data",    13, 0x0c }, //
  // xtc798.txt changes
  { "extended code 3",  15, 0x0a }, // For security messages
  { "unused",            6, 0x0b }, //
  { "extended code 2",  15, 0x0c }, // for meter read and dsm
  //
  { "status is on",     12, 0x0d }, //
  { "status is off",    13, 0x0e }, //
  { "status request",   14, 0x0f }, //
  { 0,                   0, 0xff }
};

// ===========================================================================
//
// Notes:
//
// We'll get the current string and we need to return the code value.
// We'll probably get something like this:
//
//      dimAdimAdim     (case insensitive)
//
// and we need to lookup and match the first part only, not any part in the
// string. So we need to bump the incoming string and return the code value
// so we end up with:
//      AdimAdim
//
// ===========================================================================

int
lookup(char *cmd, char *str) {
  int i = 0;
  struct aliases *x;

  x = x10aliases;

  // clprintf("Initial Cmd: %s\r\n", cmd);
  // (Ignore the case)
  while(strncasecmp(cmd, x->name, x->len) != 0) {

    if( i++ < 21) {
      ++x;
    } else {
      break;
    }
  }

  if(x->len > 0) {
    //clprintf("Match      : %s (%d)\r\n", x->name, x->len);
    strcpy(str, x->name);
  } else {
    //clprintf("No Match!\r\n");
  }

  return x->code;
}

/*
** chksum       = 16 bit checksum
**
** Checksum is derived from summing all the 8 bit data bytes in the Parameters
** and data for a serial command and taking the 2's compliment
**
** Data is everything after the command (and the checksum is 00 00)
**
** sum  = data + data + ... + data      // Sum data
** sum ~= sum;                          // Take the compliment
** sum += 1;                            // add 1
**
**                  |--> Checksum (ChkSum Hi & Lo = 0x00 0x00)
** S: => <STX> <Cmd> <Addr Lo> <Addr Hi> <Len Hi> <Len Lo> <ChkSum Hi> <ChkSum Lo> <data ...>
*/

#define _1BYTE	8

void
SendX10Addr(int device, u_char arr[], int len) {
  int i;
  word chksum = 0;

  u_char icmd[6];

  for(i = 2; i < len; i++) {
    chksum = chksum + (u_char) arr[i];
  }
  
  chksum = 0xffff - chksum + 1;

  arr[6] = (u_char) (chksum >> _1BYTE) & 0x00ff;
  arr[7] = (u_char) chksum & 0x00ff;
#if 0
  clprintf("\r\nI *** ");
  for(i = 0; i < len; i++) {
    clprintf("<0x%02x> ", arr[i]);
  }
  clprintf("chksum: 0x%04x", (word) chksum);
#endif
  write(device, arr, len);

  // send the arr
  // then send:
  icmd[0] = 0x02;
  icmd[1] = 0x46;
  icmd[2] = 0x01;
  icmd[3] = 0x66;
  icmd[4] = 0x80;
  icmd[5] = 0xf7;

  write(device, icmd, 6);
}

void
SendX10Cmd(int device, u_char arr[], int len) {
  int i;
  word chksum = 0;

  u_char icmd[6];

  for(i = 2; i < len; i++) {
    chksum = chksum + (u_char) arr[i];
  }

  chksum = 0xffff - chksum + 1;
  //clprintf("chksum: 0x%04x\r\n", chksum);

  arr[6] = (u_char) (chksum >> _1BYTE) & 0x00ff;
  arr[7] = (u_char) chksum & 0x00ff;
#if 0
  clprintf("\r\nI *** ");
  for(i = 0; i < len; i++) {
    clprintf("<0x%02x> ", (u_char) arr[i]);
  }
  clprintf("chksum: 0x%04x", (word) chksum);
#endif
  write(device, arr, len);

  // send the arr
  // then send:
  icmd[0] = 0x02;
  icmd[1] = 0x46;
  icmd[2] = 0x01;
  icmd[3] = 0x66;
  icmd[4] = 0x88;
  icmd[5] = 0xff;
#if 0
  for(i = 0; i < len; i++) {
    clprintf("<0x%02x> ", (u_char) icmd[i]);
  }
#endif
  write(device, icmd, 6);
  //clprintf("\r\n");
}

void
SendX10(int device, char *str) {
  time_t current;

  int x = 0;
  u_char hc;
  u_char uc;
  u_char code;

  u_char icmd[256];

  //clprintf("SendX10: %s\r\n", str);

  // =========================================================================
  // OK, here is where we do most of the X10 parsing. X10 commands are
  // relatively easy this to understand. They consist of 2 parts a House Code
  // <HC> which is a single letter of A - P and the second part a Key code
  // which can be either a Unit Code <UC> or a Command <CMD> string. The <UC>
  // is a single digit value, 1 - 16 (which means that when presented as a
  // string is can be either a single character, [1-9] or 2 characters 1[0-6]
  // =========================================================================
  while(*str) {
    if(isalpha(*str)) {		// (1) House code ============================
      hc = toupper(*str) - 'A';	// turn it into it's code value (a lookup idx)
      str++;			// Next character

      if(isdigit(*str)) {	// (2a) Unit code ============================
	// <HC> <UC>
	uc = *str - '0';	// turn it into it's code value (a lookup idx)
	str++;

	if(*str) {
	  if(isdigit(*str)) {
	    if(uc == 1) {
	      uc = 10 + (*str - '0');	// turn it into it's code value (a lookup idx)
	      str++;
	    } else {
	      // err! X10 only has 1 - 16
	      // clprintf("E *** Bad <UC> 0x%02x\r\n", *str);
	      // return ; 
	      // Hey, wait! This might not be an error, it could just be the
	      // next <HC>!
	    }

	  } else {
	    // Awe pity, No more strings, but we still have a string to
	    // process
	  }
	}

	uc--;		// Array offset starts at 0 not 1

	icmd[0] = 0x02;
	icmd[1] = 0x40;
	icmd[2] = 0x01;
	icmd[3] = 0x65;
	icmd[4] = 0x00;
	icmd[5] = 0x01;
	icmd[6] = 0;
	icmd[7] = 0;
	icmd[8] = (ICf(hc) <<4) + ICf(uc);

	current = time(0);	// Best we can do is 1 sec (wish I could get down to mS)
	x = current - last;	// time difference
	if(x == 0) {
	  msDelay(500);		// 1/2 a second
	}

	SendX10Addr(device, icmd, 9);
	last = time(0);

	// We do a write and now we have to wait about 1/2 a second before we can send again
      } else {			// (2b) Command ==============================
	uc = 0;			// Since it's a command it's not a <UC>
	// <HC> <CMD>
	// Lookup the command
	if((code = lookup(str, (char *) icmd)) != 0xff) {
	  x = strlen((const char *) icmd);

	  do {
	    str++;
	    x--;
	  } while(x);

	  // Process string
	} else {
	    clprintf("E *** Bad <CMD> %s\r\n", str);
	    return ;
	}
#if 0
	for(x = 0; x < 16; x++) {
	  clprintf("<0x%02x> ", (u_char) housecode[x]);
	}
	clprintf("\r\n");

	clprintf("HC = 0x%02x (%02x), code = 0x%02x, cmd = %s\r\n", ICf(hc), hc, code, icmd);
#endif
	icmd[0] = 0x02;
	icmd[1] = 0x40;
	icmd[2] = 0x01;
	icmd[3] = 0x65;
	icmd[4] = 0x00;
	icmd[5] = 0x01;
	icmd[6] = 0;
	icmd[7] = 0;
	icmd[8] = (ICf(hc) <<4) + code;

	// Send a command
	current = time(0);	// Best we can do is 1 sec (wish I could get down to mS)
	x = current - last;	// time difference
	if(x == 0) {
	  msDelay(500);		// 1/2 a second
	}

	SendX10Cmd(device, icmd, 9);
      }
    } else {

      clprintf("E *** Bad <HC> 0x%02x\r\n", *str);
      return ;
    } // =[ end if(isalpha(*str)) ]===========================================
  }
  // =[ end while(*str) ]=====================================================
}
