/*
iplcd (c) Copyright Neil Cherry 2005
Portions of this code have been taken from x10d.c and are (c) Copyright
Daniel D. Lanciani 1995-2005

All rights reserved.
 
iplcd is licensed free of charge for non-commercial distribution and for
personal and internal business use only.  iplcd may not be distributed
for profit, nor may it be included in products or otherwise distributed by
commercial entities to their clients or customers without the prior written
permission of the authors.
 
TO THE EXTENT ALLOWED BY APPLICABLE LAW, iplcd IS PROVIDED
"AS IS", WITH NO EXPRESS OR IMPLIED WARRANTY, INCLUDING, BUT
NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.
 
IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL THE AUTHOR BE
LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO
USE iplcd EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH DAMAGES.
 
These copyright, license, and disclaimer notices must be included
with all copies of iplcd.  Modified versions of iplcd must be
marked as such.

ddl@danlan.com
*/


/*
**
** Author:      Neil Cherry
** Date:        10/31/99
** Version:     1.1 (sub alpha)
**
** $Id$
**
** $Log$
**
*/

#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>


#include <errno.h>

extern int errno;

#ifdef UUCP
#include <stdlib.h>

#include "uufuncs.h"

extern int checklock(char *name);
extern int makelock(char *name);

char *lock;

#endif

extern void fini(int);

extern int errno;
extern int on;
extern int mydaemon;
extern int myusb;

#ifndef STDIN

#define STDIN	0
#define STDOUT	1
#define STDERR	2

#endif

struct termios oldsb, newsb;

/*
** Housekeeping to startup the hcsd
*/

int
setup(char *line) {
  int i, device;
  
#ifdef UUCP
  char *p;
#endif

  /*
  ** When this program comes out of Alpha I will enable these
  ** I've noticed some wierd behavior and having it print to stdout
  ** can be very useful (though disconcerting).
  */

  /*
  ** This should keep umount from saying that the device is busy
  */

  if(chdir("/")) {
    syslog(LOG_CRIT, "Can not cd to '/': %m");
    exit(errno);
  }

  /*
  ** The child process will drop into the background
  ** While the parent return to the shell (run as a daemon).
  */

  if(mydaemon) {
    close(STDIN);
    close(STDOUT);
    close(STDERR);

    if(fork()) {
      exit(0);
    }

    /*
    ** This is the POSIX (I hope) way of disassociating the process
    ** with a controlling terminal
    */

    if(setsid() < 0) {
      syslog(LOG_CRIT, "setsid error: %m");
      exit(errno);
    }
  }

  /*
  ** I should probably switch my user id over to "nobody.uucp"
  ** This is so I'm not running as root. However, I must be careful of
  ** permissions.
  */

#ifdef UUCP
  /*
  ** UUCP lock file handling
  */
  
  if((lock = malloc((sizeof(char) * MAXLINE)+1)) == NULL) {
    syslog(LOG_CRIT, "lock file malloc error: %m");
    exit(1);
  }

  /*
  ** We need a function which strips out the basename
  */

  p = rindex(line, '/');        /* Operate like strrpbrk, returns a ptr to the last / */
  p++;                          /* points to the rest of the str after the / */

  (void) sprintf(lock, LOCK, p); /* build the lock filename w/path */

  /*
  ** Check for a lock file and fail if already locked
  */

  switch(i = checklock(lock))
  {
    case !FOUND:
      if(makelock(lock) == FAIL) {
        syslog(LOG_ERR, "makelock failed");
        exit(1);
      }
      break;
    
    case FOUND:
      syslog(LOG_ERR, "Lock file has been found\n");
      exit(1);
      break;

    case OURS:
      syslog(LOG_ERR, "Lock file has been found and it is our process\n");
      unlink(lock);
      exit(1);
      break;

    case DEAD:
      if(unlink(lock)) {
        syslog(LOG_ERR, "Lock file has been found and is a dead process\n");
        exit(errno);
      } else {
        makelock(lock);
      }
      break;

    default:
      syslog(LOG_ERR, "We got an errno code back (%03x)", i);
      exit(1);
      break;
  }

#endif
  
  if((device=open(line, O_RDWR|O_NOCTTY|O_NONBLOCK)) < 0) {
    syslog(LOG_ERR, "%s: %m", line);
    exit(errno);
  }

  // =========================================================================
  //
  // This is experimental, I don't have the USB Insteon PLC so I can't test
  // this but basically if the system has my iplc device driver loaded then
  // it should behave like a serial port. If it does then this should work
  // fine.
  //
  // =========================================================================
  if(!myusb) {
    /*
    ** This works for heyu (NJC)
    */
 
    i = tcgetattr(device, &oldsb);
 
    if (i < 0) {
      p = strerror(errno);
      syslog(LOG_CRIT, p);
      exit(errno);
    }
 
    newsb = oldsb;

#if 0          
    newsb.c_iflag = IGNBRK | IGNPAR; /* INLCR - Translate NL to CR on input*/
    /*newsb.c_oflag = 0;*/
    newsb.c_oflag = ONLRET; // ???
    newsb.c_lflag = 0;
    /*newsb.c_lflag = ISIG;*/
#else
    cfmakeraw(&newsb);
#endif
    newsb.c_cflag = (CLOCAL | B4800 | CS8 | CREAD);

    for (i = 0; i < NCC; i++)
      newsb.c_cc[i] = 0;

    newsb.c_cc[VMIN]   = 1;
    newsb.c_cc[VTIME]  = 0;
 
    tcsetattr(device, TCSADRAIN, &newsb);
  }

  signal(SIGPIPE, SIG_IGN);
  signal(SIGHUP,  fini);        /* needed for uucp lock files */
  signal(SIGSEGV, fini);        /* needed for uucp lock files */
  signal(SIGFPE,  fini);        /* needed for uucp lock files */
  signal(SIGABRT, fini);        /* needed for uucp lock files */
  signal(SIGTERM, fini);        /* needed for uucp lock files */
  signal(SIGCHLD, fini);        /* The SIO failed!            */

  return(device);
}

