/*
** iplcd - daemon interface to the Insteon PLC
** Copyright (C) 2005  Neil Cherry (ncherry@comcast.net)
** http://linuxha.sourceforge.net/
** http://home.comcast.net/~ncherry/index.html
**
** This file is part of iplcd.
**
** iplcd is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** iplcd is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h>

#include "types.h"

void
SendInsteon(int device, char *str) {
  char *tptr;
  char token[256][256];
  int n = 0;
  int len = 0;

  tptr = str;

  printf("C +++ SendInsteon: [%s]\r\n", tptr); // Comment

  if(strlen(tptr) > 512) {
    printf("E *** token too long\r\n");
    return ;
  }

  // [ 00.00.01 on full]
  // eat any extra spaces at the beginning
  while(isspace(*tptr)) {
    tptr++;
  }

  // Ex: [00.00.01 on full] (extra white space, up front, is gone)
  //     parses as:
  //     000001 (non-alpha-numeric characters get skipped) may not be a good side effect!
  //     on
  //     full
  while(*tptr) {
    if(isalnum(*tptr)) {
      if(len < 255) {
	token[n][len] = *tptr;
	len++;
      } else {
	token[n][255] = 0;
	printf("E *** token too long\r\n");
	return ;
      }
      tptr++;
    } else if(isspace(*tptr)) { // next character starts a new token (or a null)
      while(isspace(*tptr)) {
	tptr++;
      }

      if(*tptr) {
	if(len < 255) {
	  token[n][len] = 0;
	} else {
	  token[n][255] = 0;
	}
	n++;
	len = 0;
      }
    } else { // skip everything else
      if(*tptr) {
	tptr++;
      } else {
	break;
      }
    } 
  }

  if(len < 255) {
    token[n][len] = 0;
  } else {
    token[n][255] = 0;
  }

  len = n;
  n = 0;
  do {
    printf("(%d) Token[%d]: %s \n", len, n, token[n]);
  } while (n++ < len);
  
}

#define ICf(p)  idcode[p]

/*
	Housecode	Device Code	Binary Value	Hex
	A		1		0110		06
	B		2		1110		0E
	C		3		0010		02
	D		4		1010		0A
	E		5		0001		01
	F		6		1001		09
	G		7		0101		05
	H		8		1101		0D
	I		9		0111		07
	J		10		1111		0F
	K		11		0011		03
	L		12		1011		0B
	M		13		0000		00
	N		14		1000		08
	O		15		0100		04
	P		16		1100		0C
*/

// House code & Unit code.
u_char idcode[] = { 0x06, 0x0E, 0x02, 0x0A,
		    0x01, 0x09, 0x05, 0x0D,
		    0x07, 0x0F, 0x03, 0x0B,
		    0x00, 0x08, 0x04, 0x0C };

u_char hcode[] = { 'M', 'E', 'C', 'K',
	           'O', 'G', 'A', 'I',
                   'N', 'F', 'D', 'L',
                   'P', 'H', 'B', 'J' 
};

struct aliases {
  char *name;
  int  len;
  int  code;
} x10aliases[] = {
  { "all units off",    13, 0x00 }, // 
  { "all lights on",    13, 0x01 }, //
  { "on",	         2, 0x02 }, //
  { "off",               3, 0x03 }, //
  { "dim",               3, 0x04 }, //
  { "bright",            5, 0x05 }, //
  { "all lights off",   14, 0x06 }, //
  { "extended code",    13, 0x07 }, //
  { "extended code 1",  15, 0x07 }, // For data/control
  { "hail request",     12, 0x08 }, //
  { "hail acknowledge", 16, 0x09 }, //
  // Preset Dim comments
  // In a Pre-Set Dim instruction, the D8 bit represents the MSB of the
  // level and H1, H2, H8, H8 represent the LSBs.
  { "preset dim 0",     12, 0x0a }, //
  { "preset dim 1",     12, 0x0b }, //
  { "extended data",    13, 0x0c }, //
  // xtc798.txt changes
  { "extended code 3",  15, 0x0a }, // For security messages
  { "unused",            6, 0x0b }, //
  { "extended code 2",  15, 0x0c }, // for meter read and dsm
  //
  { "status is on",     12, 0x0d }, //
  { "status is off",    13, 0x0e }, //
  { "status request",   14, 0x0f }, //
  { 0,                   0, 0xff }
};

// ===========================================================================
// We'll get the current string and we need to return the code value.
// We'll probably get something like this:
//
//      dimAdimAdim     (case insensitive)
//
// and we need to lookup and match the first part only, not any part in the
// string. So we need to bump the incoming string and return the code value
// ===========================================================================

int
lookup(char *cmd, char *str) {
  int i = 0;
  struct aliases *x;

  x = x10aliases;

  printf("Initial Cmd: %s\n", cmd);
  // (Ignore the case)
  while(strncasecmp(cmd, x->name, x->len) != 0) {

    if( i++ < 21) {
      ++x;
    } else {
      break;
    }
  }

  if(x->len > 0) {
    printf("Match      : %s (%d)\n", x->name, x->len);
    strcpy(str, x->name);
  } else {
    printf("No Match!\n");
  }

  return x->code;
}

/*
** chksum       = 16 bit checksum
**
** Checksum is derived from summing all the 8 bit data bytes in the Parameters
** and data for a serial command and taking the 2's compliment
**
** Data is everything after the command (and the checksum is 00 00)
**
** sum  = data + data + ... + data      // Sum data
** sum ~= sum;                          // Take the compliment
** sum += 1;                            // add 1
**
**                  |--> Checksum (ChkSum Hi & Lo = 0x00 0x00)
** S: => <STX> <Cmd> <Addr Lo> <Addr Hi> <Len Hi> <Len Lo> <ChkSum Hi> <ChkSum Lo> <data ...>
*/

void
SendX10Addr(int device, char arr[], int len) {
  int i;
  word chksum = 0;

  u_char icmd[6];

  for(i = 0; i < len; i++) {
    printf("<0x%02x> ", arr[i]);
  }

  for(i = 2; i < len; i++) {
    chksum = chksum + arr[i];
  }
  
  chksum = 0xffff - chksum + 1;
  printf("chksum: 0x%04x\n", chksum);

  // send the arr
  // then send:
  icmd[0] = 0x02;
  icmd[1] = 0x46;
  icmd[2] = 0x01;
  icmd[3] = 0x66;
  icmd[4] = 0x80;
  icmd[5] = 0xf7;

  for(i = 0; i < len; i++) {
    printf("<0x%02x> ", icmd[i]);
  }

  printf("\n");
}

void
SendX10Cmd(int device, char arr[], int len) {
  int i;
  word chksum = 0;

  u_char icmd[6];

  for(i = 0; i < len; i++) {
    printf("<0x%02x> ", arr[i]);
  }

  for(i = 2; i < len; i++) {
    chksum = chksum + arr[i];
  }
  
  chksum = 0xffff - chksum + 1;
  printf("chksum: 0x%04x\n", chksum);

  // send the arr
  // then send:
  icmd[0] = 0x02;
  icmd[1] = 0x46;
  icmd[2] = 0x01;
  icmd[3] = 0x66;
  icmd[4] = 0x88;
  icmd[5] = 0xff;

  for(i = 0; i < len; i++) {
    printf("<0x%02x> ", icmd[i]);
  }

  printf("\n");
}

void
SendX10(int device, char *str) {
  int x = 0;
  u_char hc;
  u_char uc;
  u_char code;

  char icmd[256];

  printf("SendX10: %s\r\n", str);

  // =========================================================================
  // OK, here is where we do most of the X10 parsing. X10 commands are
  // relatively easy this to understand. They consist of 2 parts a House Code
  // <HC> which is a single letter of A - P and the second part a Key code
  // which can be either a Unit Code <UC> or a Command <CMD> string. The <UC>
  // is a single digit value, 1 - 16 (which means that when presented as a
  // string is can be either a single character, [1-9] or 2 characters 1[0-6]
  // =========================================================================
  while(*str) {
    if(isalpha(*str)) {		// (1) House code ============================
      toupper(*str);
      hc = *str - 'A';		// turn it into it's code value (a lookup idx)
      str++;			// Next character

      if(isdigit(*str)) {	// (2a) Unit code ============================
	// <HC> <UC>
	uc = *str - '0';	// turn it into it's code value (a lookup idx)
	str++;
	if(*str) {
	  if(isdigit(*str)) {
	    if(uc == 1) {
	      uc  = 10;		// I really should check this that it's a 1
	      uc += *str - '0';	// turn it into it's code value (a lookup idx)
	    } else {
	      // err! X10 only has 1 - 16
	      // printf("E *** Bad <UC> 0x%02x\r\n", *str);
	      // return ;
	      // Hey, wait! This might not be an error, it could just be the
	      // next <HC>!
	    }

	  } else {
	    // Awe pity, No more strings, but we still have a string to
	    // process
	  }
	}
	uc--;		// Array offset starts at 0 not 1
	// Send an address
	// SendX10Addr(device, hc, uc);
	printf("HC = 0x%02x (%c), UC = 0x%02x (%d)\n", ICf(hc), hcode[ICf(hc)], ICf(uc), uc);

	icmd[0] = 0x02;
	icmd[1] = 0x40;
	icmd[2] = 0x01;
	icmd[3] = 0x65;
	icmd[4] = 0x00;
	icmd[5] = 0x01;
	icmd[6] = 0;
	icmd[7] = 0;
	icmd[8] = (ICf(hc) <<4) + ICf(uc);

	SendX10Addr(device, icmd, 9);
      } else {			// (2b) Command ==============================
	uc = 0;			// Since it's a command it's not a <UC>
	// <HC> <CMD>
	// Lookup the command
	if((code = lookup(str, icmd)) != 0xff) {
	  x = strlen(icmd);

	  do {
	    str++;
	    x--;
	  } while(x);

	  // Process string
	} else {
	    printf("E *** Bad <CMD> %s\r\n", str);
	    return ;
	}

	printf("HC = 0x%02x, code = 0x%02x, cmd = %s\r\n", ICf(hc), code, icmd);

	icmd[0] = 0x02;
	icmd[1] = 0x40;
	icmd[2] = 0x01;
	icmd[3] = 0x65;
	icmd[4] = 0x00;
	icmd[5] = 0x01;
	icmd[6] = 0;
	icmd[7] = 0;
	icmd[8] = (ICf(hc) <<4) + code;

	// Send a command
	SendX10Cmd(device, icmd, 9);
      }
    } else {

      printf("E *** Bad <HC> 0x%02x\r\n", *str);
      return ;
    } // =[ end if(isalpha(*str)) ]===========================================
  }
  // =[ end while(*str) ]=====================================================
}

int
hextostr(char *str) {
  int x = 0;

  while(*str) {
    if(isxdigit(*str)) { 
      x = x << 4; // half a byte 
      *str = toupper(*str);

      printf("%c ", *str);

      if(isdigit(*str)) {
	printf("");
	x += (byte) (*str - '0');
      } else {
	x += (byte) (*str - 0x37); // '' = 'A' - 10 ;
      }
    }
    printf(" (%02x) ", x);
    str++;
  }
  return x;
}

main() {
  int x;

  char *astr = " 00.00.01 on full";
  char tmp[] = "0002c2";
  char *x10 = "A1A2A3AONADIMADIMADIM";

  SendInsteon(1, astr);
  SendX10(1, x10);

  x = hextostr(tmp);
  printf("X = %d\n", x);
  return 0;
}
