/*
iplcd (c) Copyright Neil Cherry 2005
Portions of this code have been taken from x10d.c and are (c) Copyright
Daniel D. Lanciani 1995-2005

All rights reserved.
 
iplcd is licensed free of charge for non-commercial distribution and for
personal and internal business use only.  iplcd may not be distributed
for profit, nor may it be included in products or otherwise distributed by
commercial entities to their clients or customers without the prior written
permission of the authors.
 
TO THE EXTENT ALLOWED BY APPLICABLE LAW, iplcd IS PROVIDED
"AS IS", WITH NO EXPRESS OR IMPLIED WARRANTY, INCLUDING, BUT
NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.
 
IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL THE AUTHOR BE
LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO
USE iplcd EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH DAMAGES.
 
These copyright, license, and disclaimer notices must be included
with all copies of iplcd.  Modified versions of iplcd must be
marked as such.

ddl@danlan.com
*/


#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>

#include "types.h"
#include "x10.h"
#include "defs.h"

extern int  clprintf(const char *format, ...);
extern void clperror(const char *format, ...);
extern int  waitfor_rd(int fd, int nsec, int usec);
extern int  clprintf(const char *format, ...);

extern struct aliases x10aliases[];
extern char idcode[];

char dbgstr[MAXLINE];

#define X10_ADDR	0x00
#define X10_CMD		0x01

extern char *x10_keycodes[];
extern char *x10cmds[];

extern u_char housecode[];
extern u_char unitcode[];

/*
** We've already received the <STX> <CMD_X10> now we need to get the rest of
** the command which whould be <0x00> <addr> or <0x01> <cmd>
*/
word unit = 0;

void
x10read(int device) {
  int rtn;
  int cnt = 3;
  int hc, uc;

  u_char xbuf[10];

  // wait for read
  waitfor_rd(device, 0, 2080); // 4800 = 208 us/bit,multi by 10 bits = 2080
  // Read character
  do {
    errno = 0;
    if((( rtn = read(device, xbuf, 1)) < 1) || errno != 0) {
      if(errno != 11) {
	// We ignore timeouts, happens if we have no data
	clprintf("\r\nE *** <ERR> (%d/%d) ", rtn, errno);
	clperror("\r\nE *** Read error (0x%02x)", *xbuf);
      }
    } else {
      break;
    }
  } while(--cnt);

  if(*xbuf == X10_ADDR) {
  // if 0x00 it's an address
  //   wait for read
  //   switch
  //     case ...
    waitfor_rd(device, 0, 2080); // 4800 = 208 us/bit,multi by 10 bits = 2080

    // Read character
    do {
      errno = 0;
      if((( rtn = read(device, xbuf, 1)) < 1) || errno != 0) {
	// We ignore timeouts, happens if we have no data
	clprintf("\r\nE *** <ERR> (%d/%d) ", rtn, errno);
	clperror("\r\nE *** Read error (0x%02x)", *xbuf);
      } else {
	break;
      }
    } while(--cnt);
    hc = (*xbuf & 0xf0) >> 4;
    uc = (*xbuf & 0x0f);
    //clprintf("\r\nR X10 addr:0x%02x (%c) (%s)", *xbuf, housecode[hc], x10_keycodes[uc]);
    // The insteon PLC returns:
    //
    //    0x02 0x45 0x08
    //    0x02 0x4A 0x00 0x66
    //    
    //    <STX> Event 0x45 0x08
    //    <STX> R X10 addr:0x66 (A) (Unit 1)
    //    
    //    0x02 0x45 0x08
    //    0x02 0x4A 0x01 0x63
    //    
    //    <STX> Event 0x45 0x08
    //    <STX> R X10 Cmd:0x63 (A) (Off)
    //
    //    a1aoff
    //
    //    R A01 1 2 Unit 1
    //    R A20 1 2 off
    //    
    // In x10d (Dan Lanciani's x10 daemon) if you type:
    //
    //    a3adim (case insensitive)
    //
    // you get back:
    //
    //    R A03 4 2 Unit 3
    //    R A21 4 2 Dim
    //
    // So it appers that it's broken down as follows:
    //    R      - Response
    //    A      - The house code
    //    03     - Index into x10longnames array
    //    4      - Device (UC) 3 is 0100 (4). Could be bits such as with A1A2A3
    //             which would give us 0111 = 7
    //    2      - Repeat count, 2 is normal but more are possible.
    //    Unit 3 - The string stored at x10longnames[3]
    //
    // It looks like the Insteon PLC doesn't handle Extended commands
    //
    // 1 << (unitcode[uc] - 1) looks ok for masking this unit but we need to add
    // the rest of the units that are involved with this command.
    //
    unit = unit | (1 << (unitcode[uc] - 1));
    clprintf("\r\nR %c%02d %d 2 %s", housecode[hc], unitcode[uc], unit, x10_keycodes[uc]);
  } else if(*xbuf == X10_CMD) {
  // if 0x01 it's a command
  //   wait for read
  //   switch
  //     case ...

    waitfor_rd(device, 0, 2080); // 4800 = 208 us/bit,multi by 10 bits = 2080

    // Read character
    do {
      errno = 0;
      if((( rtn = read(device, xbuf, 1)) < 1) || errno != 0) {
	// We ignore timeouts, happens if we have no data
	clprintf("\r\nE *** (%d/%d) ", rtn, errno);
	clperror("\r\nE *** Read error (0x%02x)", *xbuf); 
      } else {
	break;
      }
    } while(--cnt);
    hc = (*xbuf & 0xf0) >> 4;
    uc = (*xbuf & 0x0f);
    clprintf("\r\nR %c%02d %d 2 %s\r\n", housecode[hc], uc+17, unit, x10cmds[uc]);
    unit = 0;
  } else {
    clprintf("E other :0x%02x\r\n", *xbuf);
  // otherwise ???
  //   ???
  }
}

// =[ x10 write routines ]====================================================

