/*
iplcd (c) Copyright Neil Cherry 2005
Portions of this code have been taken from x10d.c and are (c) Copyright
Daniel D. Lanciani 1995-2005

All rights reserved.
 
iplcd is licensed free of charge for non-commercial distribution and for
personal and internal business use only.  iplcd may not be distributed
for profit, nor may it be included in products or otherwise distributed by
commercial entities to their clients or customers without the prior written
permission of the authors.
 
TO THE EXTENT ALLOWED BY APPLICABLE LAW, iplcd IS PROVIDED
"AS IS", WITH NO EXPRESS OR IMPLIED WARRANTY, INCLUDING, BUT
NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE.
 
IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL THE AUTHOR BE
LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO
USE iplcd EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY
OF SUCH DAMAGES.
 
These copyright, license, and disclaimer notices must be included
with all copies of iplcd.  Modified versions of iplcd must be
marked as such.

ddl@danlan.com
*/

#include "types.h"

// ===========================================================================
// This gets a bit complicated! House/Key codes are 9 bits long. So how do
// we handle this, well it turns out that the fix is simple enough. D16 goes
// into the cmd/address byte. So an a1aon get turned to 4 bytes: 00 66 01 62
// Translated:
//  00 = address (D16 = 0)
//  66 = A (6) 1 (6)
//  01 = Command (D16 = 1)
//  62 = A (6) On (2)
//
// HC H1 H2 H3 H4  KC D1 D2 D4 D8 D16
// A  0  1  1  0   1  0  1  1  0  0
// A  0  1  1  0  On  0  0  1  0  1
//
// Now for part II of the confusion; in 1993 X10 revised the X10 commands
// (Document xtc798.txt) and changed the Preset dim 0, 1 and extended data to
//  extended code 3, unused and extended code 2. I think Insteon & Smarthome
// both use Preset commands (which they've been using since before '98.
// ===========================================================================

/*
	Housecode	Device Code	Binary Value	Hex
	M		13		0000		00
	E		5		0001		01
	C		3		0010		02
	K		11		0011		03
	O		15		0100		04
	G		7		0101		05
	A		1		0110		06
	I		9		0111		07
	N		14		1000		08
	F		6		1001		09
	D		4		1010		0A
	L		12		1011		0B
	P		16		1100		0C
	H		8		1101		0D
	B		2		1110		0E
	J		10		1111		0F

*/

u_char housecode[] = { 'M', 'E', 'C', 'K',
		       'O', 'G', 'A', 'I',
		       'N', 'F', 'D', 'L',
		       'P', 'H', 'B', 'J' 
};

u_char unitcode[] = { 13,  5,  3, 11,
		      15,  7,  1,  9,
		      14,  6,  4, 12,
		      16,  8,  2, 10 
};

/*
	Housecode	Device Code	Binary Value	Hex
	A		1  		0110		06
	B		2		1110		0E
	C		3		0010		02
	D		4		1010		0A
	E		5		0001		01
	F		6		1001		09
	G		7		0101		05
	H		8		1101		0D
	I		9		0111		07
	J		10		1111		0F
	K		11		0011		03
	L		12		1011		0B
	M		13		0000		00
	N		14		1000		08
	O		15		0100		04
	P		16		1100		0C
*/

// House code & Unit code.
u_char idcode[] = { 0x06, 0x0E, 0x02, 0x0A,
		    0x01, 0x09, 0x05, 0x0D,
		    0x07, 0x0F, 0x03, 0x0B,
		    0x00, 0x08, 0x04, 0x0C
};

char *x10_keycodes[] = {
  "Unit 13",
  "Unit 3",
  "Unit 5",
  "Unit 11",
  "Unit 15",
  "Unit 7",
  "Unit 1",
  "Unit 9",
  "Unit 14",
  "Unit 6",
  "Unit 4",
  "Unit 12",
  "Unit 16",
  "Unit 8",
  "Unit 2",
  "Unit 10"
};

char *x10cmds[] = {
  "All Units Off",	// 0
  "All Lights On",	// 1
  "On",			// 2
  "Off",		// 3
  "Dim",		// 4
  "Bright",		// 5
  "All Lights Off",	// 6
  "Extended Code",	// 7
  "Hail Request",	// 8
  "Hail Acknowledge", 	// 9
  "Preset Dim 0",	// 10
  "Preset Dim 1",	// 11
  "Extended Data",	// 12
  "Status = On",	// 13
  "Status = Off",	// 14
  "Status Request"	// 15
};
